
#include "Resources.h"
#include "../utils/DXHelpers.h"
#include "../Vertices.h"
#include <cassert>

CResources::CResources()
:	mTexEnvi(NULL), mTexBowl(NULL), mTexPoint(NULL),
	mEfxWater(NULL), mEfxParticle(NULL), mEfxBaseTexEnvi(NULL), mEfxBlendEnvi(NULL),
	mMeshBowl(NULL), mMeshWater(NULL), mMeshWaterSystem(NULL), mMeshWaterDynamic(NULL)
{
}

void CResources::init( CFluid& fluid )
{
	mFluid = &fluid;
}
	


HRESULT CResources::onCreateDevice( IDirect3DDevice8* device )
{
	CDXObject::onCreateDevice( device );
	
	// load textures
	dx::loadTexture( mTexEnvi, getDevice(), "tex/Environment.png" );
	dx::loadTexture( mTexPoint, getDevice(), "tex/WaterPoint.png" );
	dx::loadTexture( mTexBowl, getDevice(), "tex/Puodas.jpg" );
	
	// load effects
	dx::loadEffect( mEfxWater, getDevice(), "efx/Water.sha" );
	dx::loadEffect( mEfxParticle, getDevice(), "efx/Particle.sha" );
	dx::loadEffect( mEfxBaseTexEnvi, getDevice(), "efx/BaseTexEnvi.sha" );
	dx::loadEffect( mEfxBlendEnvi, getDevice(), "efx/BlendEnvi.sha" );
	
	// load meshes
	dx::loadMesh( mMeshBowl, getDevice(), "mesh/Puodas.x", D3DXMESH_MANAGED, &mMaterialBowl );
	dx::loadMesh( mMeshWater, getDevice(), "mesh/Vanduo.x", D3DXMESH_SYSTEMMEM, &mMaterialWater );
	mMeshWater->CloneMeshFVF( D3DXMESH_SYSTEMMEM, FVF_XYZ, &getDevice(), &mMeshWaterSystem );
	
	return S_OK;
}

HRESULT CResources::onLostDevice()
{
	CDXObject::onLostDevice();

	assert( mEfxWater );
	mEfxWater->OnLostDevice();
	assert( mEfxParticle );
	mEfxParticle->OnLostDevice();
	assert( mEfxBaseTexEnvi );
	mEfxBaseTexEnvi->OnLostDevice();
	assert( mEfxBlendEnvi );
	mEfxBlendEnvi->OnLostDevice();

	dx::assertRelease( mMeshWaterDynamic );
	
	return S_OK;
}

HRESULT CResources::onResetDevice()
{
	CDXObject::onResetDevice();

	assert( mEfxWater );
	mEfxWater->OnResetDevice();
	dx::findValidTechnique( mEfxWater );
	mEfxWater->SetTexture( "texParticle", mTexPoint );
	assert( mEfxParticle );
	mEfxParticle->OnResetDevice();
	dx::findValidTechnique( mEfxParticle );
	mEfxParticle->SetTexture( "texBase", mTexPoint );
	assert( mEfxBaseTexEnvi );
	mEfxBaseTexEnvi->OnResetDevice();
	dx::findValidTechnique( mEfxBaseTexEnvi );
	mEfxBaseTexEnvi->SetTexture( "texBase", mTexBowl );
	mEfxBaseTexEnvi->SetTexture( "texEnvi", mTexEnvi );
	assert( mEfxBlendEnvi );
	mEfxBlendEnvi->OnResetDevice();
	dx::findValidTechnique( mEfxBlendEnvi );
	mEfxBlendEnvi->SetTexture( "texEnvi", mTexEnvi );
	
	assert( !mMeshWaterDynamic );
	mMeshWater->CloneMeshFVF( D3DXMESH_DYNAMIC | D3DXMESH_WRITEONLY, FVF_XYZ_NORMAL, &getDevice(), &mMeshWaterDynamic );
	assert( mMeshWaterDynamic );
	
	return S_OK;
}

HRESULT CResources::onDestroyDevice()
{
	CDXObject::onDestroyDevice();
	
	// release textures
	dx::assertRelease( mTexEnvi );
	dx::assertRelease( mTexPoint );
	dx::assertRelease( mTexBowl );
	
	// release effects
	dx::assertRelease( mEfxWater );
	dx::assertRelease( mEfxParticle );
	dx::assertRelease( mEfxBaseTexEnvi );
	dx::assertRelease( mEfxBlendEnvi );
	
	// release meshes
	dx::assertRelease( mMeshBowl );
	dx::assertRelease( mMeshWater );
	dx::assertRelease( mMeshWaterSystem );

	return S_OK;
}
